/*
 * file: XxCore.cpp
 *
 * This file is part of the XxWinLib library which is developed to support
 * the development of NetStreamer. This file is distributed under the
 * GNU GENERAL PUBLIC LICENSE, see the accompanying COPYING file.
 *
 * Copyright (C) 1997 Rolf Fokkens
 *
 */

#include <iostream.h>

#include "XxCore.h"
#include "BiLink.h"
#include "XxAutoInit.h"

#include "XxApplication.h"

static XxCore::XxCoreSet::SetOwner *pPreFlushOwner = NULL;

static int CoreCount = 0;

class InitClass {
private:
    int Dummy;
public:
    InitClass (void) {
        XxCore::Init ();
    };
};

AUTOINIT InitClass InitObject;

void XxCore::Init (void)
{
    if (pPreFlushOwner == NULL) {
        XxBase::Init ();
        XxApplication::Init ();
        pPreFlushOwner = new XxCore::XxCoreSet::SetOwner;
        AUTOINIT_TRACE ("XxCore::Init");
    };
};

void XxCore::MarkPreFlush ()
{
    PreFlushMember.Attach (*pPreFlushOwner);
};

void XxCore::OnPreFlush ()
{
    cout << "Unhandled OnPreFlush" << endl;
};

XxCore::XxCore (int DoCount)
    : PreFlushMember (this)
{
    XxCore::DoCount = DoCount;

    if (DoCount) CoreCount++;
};

XxCore::~XxCore (void)
{
    if (DoCount) CoreCount--;
};

void XxCore::DoPreFlushActions (void)
{
    XxCore *pBase;

    for (;;) {
        pBase = pPreFlushOwner->GetFirst ();
        if (pBase == NULL) break;
        pBase->OnPreFlush ();
        pBase->PreFlushMember.Detach ();
    };
};

Display *XxCore::GetDisplay (void)
{
    return XxApplication::GetDisplay ();
};

int XxCore::GetScreen  (void)
{
    return XxApplication::GetScreen ();
};

int XxCore::GetWidth   (void)
{
    return XxApplication::GetWidth  ();
};

Window XxCore::GetRootWindow (void)
{
    return XxApplication::GetRootWindow ();
};

int XxCore::GetDepth (void)
{
    return XxApplication::GetDepth ();
};

Atom XxCore::GetWmDeleteWindow (void)
{
    return XxApplication::GetWmDeleteWindow ();
};

int XxCore::GetCount (void)
{
    return CoreCount;
};


/*
long XxCore::GetBGW (int i)
{
    return XxApplication::GetBGW (i);
};
*/

